#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
	/// version 2.0 updated by Big Mike (www.bigmiketrading.com) on June 20 2009
    /// For DoubleMA use
	/// Variable moving average
	/// Original version by Rollins
    /// v 1.0 - 7/3/2009 (systrader): Branched version for Sharky
    /// </summary>
    [Description("DoubleMA (internal) - Sharky Package v1.0")]
    [Gui.Design.DisplayName("DoubleMA (internal Sharky Pkg)")]
    public class DoubleMA_internal_SharkyPkg : Indicator
    {
         #region Variables
        // Wizard generated variables
            private int mAPeriod = 1; // Default setting for MAPeriod
			private DMAType mAType = DMAType.EMA;
			private int typeInt;
			private bool bInit = false;
			public enum DMAType			// A list of the valid types of moving averages that may be selected.
			{
				EMA = 1,					// Exponential Moving Average.
				HMA = 2,					// Hull Moving Average.
				WMA = 3,					// Weighted Moving Average.
				SMA = 4,					// Simple Moving Average.
				MAX = 5,					// Weighted Moving Average.
				MIN = 6,					// Weighted Moving Average.
				VMA = 7,
				Wilder = 8,
				EhlersFilter = 9,
				ADXVMA = 10,
				ZeroLagEMA = 11,
				TMA = 12,
				VWMA = 13,
				SMMA = 14,
				JMA = 15,
				TEMA = 16,
				ZeroLagTEMA = 17,
				ZeroLagHATEMA = 18,				
				None = 16					// Weighted Moving Average.
				
			}
			Indicator indicator;
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			try
			{
				Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "MA"));
				Overlay				= true;
				PriceTypeSupported	= true;

			}
			catch( Exception ex )
			{
				Print( ex.ToString() );
			}
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			try
			{
				
				if( mAType == DMAType.EMA )
					MA.Set( EMA(Input,MAPeriod)[0] );
				else if( mAType == DMAType.HMA )
					MA.Set( HMA(Input,MAPeriod)[0] );
				else if( mAType == DMAType.WMA )
					MA.Set( WMA(Input,MAPeriod)[0] );
				else if( mAType == DMAType.SMA )
					MA.Set( SMA(Input,MAPeriod)[0] );
				else if( mAType == DMAType.MAX )
					MA.Set( MAX(Input,MAPeriod)[0] );
				else if( mAType == DMAType.MIN )
					MA.Set( MIN(Input,MAPeriod)[0] );
				else if( mAType == DMAType.VMA )
					MA.Set( VMA(Input,MAPeriod / 2, MAPeriod)[0] );
				else if( mAType == DMAType.Wilder )
					MA.Set( Wilder(Input,MAPeriod)[0] );
				else if( mAType == DMAType.EhlersFilter )
					MA.Set( EhlersFilter(Input,MAPeriod)[0] );
				else if( mAType == DMAType.ADXVMA )
					MA.Set( ADXVMA_internal_SharkyPkg(Input,MAPeriod)[0] );
				else if( mAType == DMAType.ZeroLagEMA )
					MA.Set( ZeroLagEMA(Input,MAPeriod)[0] );
				else if( mAType == DMAType.TMA )
					MA.Set( TMA(Input,MAPeriod)[0] );
				else if( mAType == DMAType.VWMA )
					MA.Set( VWMA(Input,MAPeriod)[0] );
				else if( mAType == DMAType.SMMA )
					MA.Set( SMMA(Input,MAPeriod)[0] );
				else if( mAType == DMAType.JMA )
					MA.Set( JMA(Input,0,MAPeriod)[0] );
				else if( mAType == DMAType.TEMA )
					MA.Set( TEMA(Input, MAPeriod)[0] );
				else if( mAType == DMAType.ZeroLagTEMA )
					MA.Set( ZeroLagTEMA(Input, MAPeriod)[0] );
				else if( mAType == DMAType.ZeroLagHATEMA )
					MA.Set( ZeroLagHATEMA(Input, MAPeriod)[0] );				
				else
					MA.Set( Input[ 0 ] ); 
				
			}
			catch( Exception ex )
			{
				Print( ex.ToString() );
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MA
        {
            get { return Values[0]; }
        }

        [Description("")]
        [Category("Parameters")]
        public int MAPeriod
        {
            get { return mAPeriod; }
            set { mAPeriod = Math.Max(1, value); }

        }

        [Category("Parameters")]
		public int TypeInt
		{
			get { return typeInt; }
            set { 
					//typeInt = Math.Max( 1, Math.Min( 14, value ) );
					typeInt = value;
					if( typeInt == (int)DMAType.EMA )
						mAType = DMAType.EMA;
					else if( typeInt == (int)DMAType.HMA )
						mAType = DMAType.HMA;
					else if( typeInt == (int)DMAType.WMA )
						mAType = DMAType.WMA;
					else if( typeInt == (int)DMAType.SMA )
						mAType = DMAType.SMA;
					else if( typeInt == (int)DMAType.MAX )
						mAType = DMAType.MAX;
					else if( typeInt == (int)DMAType.MIN )
						mAType = DMAType.MIN;
					else if( typeInt == (int)DMAType.VMA )
						mAType = DMAType.VMA;
					else if( typeInt == (int)DMAType.Wilder )
						mAType = DMAType.Wilder;
					else if( typeInt == (int)DMAType.EhlersFilter )
						mAType = DMAType.EhlersFilter;
					else if( typeInt == (int)DMAType.ADXVMA )
						mAType = DMAType.ADXVMA;
					else if( typeInt == (int)DMAType.ZeroLagEMA )
						mAType = DMAType.ZeroLagEMA;
					else if( typeInt == (int)DMAType.TMA )
						mAType = DMAType.TMA;
					else if( typeInt == (int)DMAType.VWMA )
						mAType = DMAType.VWMA;
					else if( typeInt == (int)DMAType.SMMA )
						mAType = DMAType.SMMA;
					else if( typeInt == (int)DMAType.JMA )
						mAType = DMAType.JMA;
					
					else
						mAType = DMAType.None;
				}
		}
		#endregion
    }
	
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DoubleMA_internal_SharkyPkg[] cacheDoubleMA_internal_SharkyPkg = null;

        private static DoubleMA_internal_SharkyPkg checkDoubleMA_internal_SharkyPkg = new DoubleMA_internal_SharkyPkg();

        /// <summary>
        /// DoubleMA (internal) - Sharky Package v1.0
        /// </summary>
        /// <returns></returns>
        public DoubleMA_internal_SharkyPkg DoubleMA_internal_SharkyPkg(int mAPeriod, int typeInt)
        {
            return DoubleMA_internal_SharkyPkg(Input, mAPeriod, typeInt);
        }

        /// <summary>
        /// DoubleMA (internal) - Sharky Package v1.0
        /// </summary>
        /// <returns></returns>
        public DoubleMA_internal_SharkyPkg DoubleMA_internal_SharkyPkg(Data.IDataSeries input, int mAPeriod, int typeInt)
        {
            checkDoubleMA_internal_SharkyPkg.MAPeriod = mAPeriod;
            mAPeriod = checkDoubleMA_internal_SharkyPkg.MAPeriod;
            checkDoubleMA_internal_SharkyPkg.TypeInt = typeInt;
            typeInt = checkDoubleMA_internal_SharkyPkg.TypeInt;

            if (cacheDoubleMA_internal_SharkyPkg != null)
                for (int idx = 0; idx < cacheDoubleMA_internal_SharkyPkg.Length; idx++)
                    if (cacheDoubleMA_internal_SharkyPkg[idx].MAPeriod == mAPeriod && cacheDoubleMA_internal_SharkyPkg[idx].TypeInt == typeInt && cacheDoubleMA_internal_SharkyPkg[idx].EqualsInput(input))
                        return cacheDoubleMA_internal_SharkyPkg[idx];

            DoubleMA_internal_SharkyPkg indicator = new DoubleMA_internal_SharkyPkg();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.MAPeriod = mAPeriod;
            indicator.TypeInt = typeInt;
            indicator.SetUp();

            DoubleMA_internal_SharkyPkg[] tmp = new DoubleMA_internal_SharkyPkg[cacheDoubleMA_internal_SharkyPkg == null ? 1 : cacheDoubleMA_internal_SharkyPkg.Length + 1];
            if (cacheDoubleMA_internal_SharkyPkg != null)
                cacheDoubleMA_internal_SharkyPkg.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheDoubleMA_internal_SharkyPkg = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// DoubleMA (internal) - Sharky Package v1.0
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DoubleMA_internal_SharkyPkg DoubleMA_internal_SharkyPkg(int mAPeriod, int typeInt)
        {
            return _indicator.DoubleMA_internal_SharkyPkg(Input, mAPeriod, typeInt);
        }

        /// <summary>
        /// DoubleMA (internal) - Sharky Package v1.0
        /// </summary>
        /// <returns></returns>
        public Indicator.DoubleMA_internal_SharkyPkg DoubleMA_internal_SharkyPkg(Data.IDataSeries input, int mAPeriod, int typeInt)
        {
            return _indicator.DoubleMA_internal_SharkyPkg(input, mAPeriod, typeInt);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// DoubleMA (internal) - Sharky Package v1.0
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DoubleMA_internal_SharkyPkg DoubleMA_internal_SharkyPkg(int mAPeriod, int typeInt)
        {
            return _indicator.DoubleMA_internal_SharkyPkg(Input, mAPeriod, typeInt);
        }

        /// <summary>
        /// DoubleMA (internal) - Sharky Package v1.0
        /// </summary>
        /// <returns></returns>
        public Indicator.DoubleMA_internal_SharkyPkg DoubleMA_internal_SharkyPkg(Data.IDataSeries input, int mAPeriod, int typeInt)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DoubleMA_internal_SharkyPkg(input, mAPeriod, typeInt);
        }

    }
}
#endregion
